#include "common.as"

class CPlayer : IEntityController
{
	CPlayer(CEntity @ent)
	{
		@entity      = ent;
		entity.name  = 'player';
		moveTo = entity.position;
	}
 
	void onThink()
	{
		// Something woke us up, let's check if it is time to set a new movement target
		if( entity.position != moveTo )
		{
			vector2 to = game.GetNearestPathNode(entity.position, moveTo);
			entity.MoveTo(to, 5);
			entity.rotation = AngleFromVector(entity.velocity);
		}
		
		if( entity.GetTimer(0) < 0 )
			entity.SetTimer(0.2, 0);
		
		// Go back to sleep
		entity.awake = false;
	}
	
	void onMessage(uint msgId, ref @arg)
	{
		switch( msgId )
		{
		case msgSetWayPoint:
			{
				// We've received a new target waypoint from the user. Let's move there
				CWayPoint @wp = cast<CWayPoint>(arg);
				moveTo = vector2(wp.x, wp.y);
				vector2 to = game.GetNearestPathNode(entity.position, moveTo);
				entity.MoveTo(to, 5);
				entity.rotation = AngleFromVector(entity.velocity);
				entity.awake = false;
			}
			break;
		}
	}
	
	void onCollide(const CEntity @object, const vector2 &in position, const vector2 &in normal, float appliedImpulse)
	{
		// Oops, we hit something. Continue our movement
		if( (entity.position - moveTo).length() > 0.5 )
		{
			vector2 to = game.GetNearestPathNode(entity.position, moveTo);
			entity.MoveTo(to, 5);
			entity.rotation = AngleFromVector(entity.velocity);
			entity.awake = false;
		}
		else
		{
			// We're close enough. Let's stop and take a nap
			moveTo = entity.position;
			entity.velocity = vector2(0,0);
			entity.awake = false;
		}
	}
	
	void onTimer(int id)
	{
		const CEntity @target;
		float bestDist = 11;
	
		// Check for enemies within range
		// TODO: This function could possibly do the ray cast internally and only return those objects that are visible
		array<const CEntity @> @ents = game.FindEntitiesWithinRange(entity.position, 10);
		for( uint n = 0; n < ents.length(); n++ )
		{
			if( ents[n] !is entity && ents[n].type == 'enemy' )
			{
				if( target is null )
				{
					@target = ents[n];
					bestDist = (target.position - entity.position).length();
					
					// Check if visible
					const CEntity @obj;
					vector2 vel;
					game.RayCast(entity, 1, entity.position, target.position, obj, vel);
					if( obj !is target )
					{
						bestDist = 11;
						@target = null;
					}
				}
				else
				{
					float dist = (ents[n].position - entity.position).length();
					if( dist < bestDist )
					{
						// Check if visible
						const CEntity @obj;
						vector2 vel;
						game.RayCast(entity, 1, entity.position, ents[n].position, obj, vel);
						if( obj is ents[n] )
						{
							@target = ents[n];
							bestDist = dist;
						}
					}
				}
			}
		}
		
		if( target !is null )
		{
			// Turn towards the target
			entity.rotation = AngleFromVector(target.position - entity.position);
	
			// Fire a shot
			vector2 direction(cos(entity.rotation + pi/2), sin(entity.rotation + pi/2));
			game.SpawnEntity("bullet", entity.position + 0.5*direction, direction*200);

			// TODO: Implement an IncrementProperty(string, modify);
			uint shotsFired;
			game.GetProperty('shotsFired', shotsFired);
			game.SetProperty('shotsFired', shotsFired+1);
		}
		else
		{
			// Turn in the direction we're moving
			if( entity.velocity.lengthSq() > 0 )
				entity.rotation = AngleFromVector(entity.velocity);
		}
		
		entity.SetTimer(0.2, 0);
	}

	CEntity @entity;
	vector2 moveTo;
}