
enum TILESET
{
  WALL = 0,
  FLOOR = 1
}

CGenerator @gen;

void Generate(CGenerator @g)
{
    @gen = g;

	gen.SetMapSize(50,50);

	// Generate the rooms
	array<Room@> rooms;
	for( uint n = 0; n < 20; n++ )
	{
		Room room();
		rooms.insertLast(room);
		room.Place();
	}
	
	// Connect the rooms that are closest together
	for( uint n = 0; n < rooms.length()-1; n++ )
	{
		Room @bestRoom = null;
		int bestDist = 1000;
		for( uint m = n+1; m < rooms.length(); m++ )
		{
			int dist = rooms[n].DistanceTo(rooms[m]);
			if( dist < bestDist )
			{
				bestDist = dist;
				@bestRoom = rooms[m];
			}
		}
		
		rooms[n].CorridorTo(bestRoom);
	}
	
	gen.AddObject("player", rooms[0].x+0.5*rooms[0].w, rooms[0].y+0.5*rooms[0].h, 0, null);
	Room @room = rooms[rooms.length()-1];
	gen.AddObject("exit", room.x + 0.5*room.w, room.y + 0.5*room.h, 0, null);
	
	// Place enemies in each room, except the first and last
	for( uint n = 1; n < rooms.length()-2; n++ )
		gen.AddObject("enemy", rooms[n].x+0.5*rooms[n].w, rooms[n].y+0.5*rooms[n].h, 0, null);
}

class Room
{
	Room()
	{
		w = 3+uint(gen.GetRandomNumber()*3);
		h = 3+uint(gen.GetRandomNumber()*3);
		
		x = 1+uint(gen.GetRandomNumber()*(48-w));
		y = 1+uint(gen.GetRandomNumber()*(48-h));
	}
	
	void Place()
	{
		gen.FillRect(x,y,w,h,FLOOR);
	}
	
	int DistanceTo(Room @other)
	{
		int x1 = x+w/2;
		int y1 = y+h/2;
		int x2 = other.x+other.w/2;
		int y2 = other.y+other.h/2;
		
		// Manhattan distance
		return (x2>x1 ? x2-x1 : x1-x2) + (y2>y1 ? y2-y1 : y1-y2);
	}
	
	void CorridorTo(Room @other)
	{
		uint x1 = x+w/2;
		uint y1 = y+h/2;
		uint x2 = other.x+other.w/2;
		uint y2 = other.y+other.h/2;
		
		if( x2 < x1 ) { uint tmp = x1; x1 = x2; x2 = tmp; tmp = y1; y1 = y2; y2 = tmp; }
		
		if( y1 < y2 )
		{
			gen.FillRect(x1, y1, x2-x1 + 1, 1, FLOOR);
			gen.FillRect(x2, y1, 1, y2-y1 + 1, FLOOR);
		}
		else
		{
			gen.FillRect(x1, y1, x2-x1 + 1, 1, FLOOR);
			gen.FillRect(x2, y2, 1, y1-y2 + 1, FLOOR);
		}
	}
	
	uint x;
	uint y;
	uint w;
	uint h;
}