shared float4x4 g_mWorld : WORLD;      // World matrix
shared float4x4 g_mView  : VIEW;       // View matrix
shared float4x4 g_mProj  : PROJECTION; // Projection matrix

texture  g_txScene         : TEXTURE0;

sampler2D g_samScene =
sampler_state
{
    Texture = <g_txScene>;
    MinFilter = Linear; 
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
    float2 TextureUV  : TEXCOORD0;  // vertex texture coords 
    float4 Diffuse    : COLOR0;     // vertex color
};
VS_OUTPUT VertScene( float4 vPos         : POSITION,
                     float4 vColor       : COLOR0,
                     float2 vTex0        : TEXCOORD0 )
{
    VS_OUTPUT Out;

    // Transform the position from object space to homogeneous projection space
    Out.Position = mul( vPos, g_mWorld );
    Out.Position = mul( Out.Position, g_mView );
    Out.Position = mul( Out.Position, g_mProj );

    // Just copy the texture coordinate and vertex color through
    Out.TextureUV = vTex0;
    Out.Diffuse   = vColor;

    return Out;
}


float4 PixScene( VS_OUTPUT In ) : COLOR0
{
    // Lookup color from texture
    return In.Diffuse * tex2D(g_samScene, In.TextureUV);
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique RenderScene
{
    pass P0
    {
/*
        // Additive blending
		AlphaBlendEnable = true;
        SrcBlend = ONE;
        DestBlend = ONE;
*/
		// Alpha transparency
		AlphaBlendEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;

        // Turn off z write
        ZWriteEnable = false;

		// Turn off culling
		CullMode = NONE;
			
		// Set the vertex and pixel shaders
        VertexShader = compile vs_2_0 VertScene();
        PixelShader  = compile ps_2_0 PixScene();
    }
}
