#include "common.as"
#include "camera.as"

const float walkSpeed      = 10;
const float jumpSpeed      = 20;
const float attackTime     = 0.3;
const float invincibleTime = 2;
const float idleTime       = 3;

// TODO: The camera should be controlled by the level script
CCamera cam;

enum ETimers
{
	canAttackAgain,
	makeVulnerable
}

class CPlayer : IEntityController
{
	CPlayer(CEntity @ent)
	{
		@entity      = ent;
		entity.name  = 'player';
		isAttacking  = false;
		isInvincible = false;
		direction    = 0;
	}
 
	void onThink()
	{
		// TODO: To jump down from a platform, the player should 
		//       set the collision mask to not collide with platforms, 
		//       and then set a timer to turn on the collisions again
		//       as the movement has taken him down under the platform

		vector2 vel = entity.velocity;

		// Am I on the ground? If there is ground 5 cm below me, I consider it yes
		vector2 groundVel;
		bool onGround = game.RayCast(entity, 1, entity.position, entity.position - vector2(0, 1.05), null, groundVel) < 1;
		
		float dy = 0;
		bool walking = false;
		if( input.GetActionState(actionLeft) )
		{
			walking = true;
			if( direction >= 0 )
				direction--;
		}
		if( input.GetActionState(actionRight) )
		{
			walking = true;
			if( direction <= 0 )
				direction++;
		}
		if( input.GetActionState(actionJump) && onGround )
		{
			dy = jumpSpeed;
		}
		if( input.GetActionState(actionUse) && !isAttacking && direction != 0 )
		{
			// Did we hit something with the sword?
			vector2 tmp;
			const CEntity @obj;
			if( direction > 0 )
			{
				game.RayCast(entity, 0, entity.position + vector2(0, -0.25), entity.position + vector2(2, -0.25), obj, tmp);
				entity.SetAnimation("attack_r");
			}
			else if( direction < 0 )
			{
				game.RayCast(entity, 0, entity.position + vector2(0, -0.25), entity.position + vector2(-2, -0.25), obj, tmp);
				entity.SetAnimation("attack_l");
			}
			if( obj !is null )
			{
				// Touch it. This will tell the other object we're hitting it
				entity.Touch(obj);
			}
			// TODO: The sound volume should be set by the sfx file
			entity.PlaySound('slash',0.05);
			
			// Set timer to let us attack again
			isAttacking = true;
			entity.SetTimer(attackTime, canAttackAgain);
		}

		if( !isAttacking )
		{
			float x = walking ? direction * walkSpeed : 0;
			if( abs(x) > 0.001 )
			{
				if( direction > 0 )
					entity.SetAnimation("walk_r");
				else
					entity.SetAnimation("walk_l");
			}
			else
			{
				if( direction == 0 )
					entity.SetAnimation("idle");
				else if( direction > 0 )
					entity.SetAnimation("still_r");
				else if( direction < 0 )
					entity.SetAnimation("still_l");
			}
			if( !onGround )
			{
				if( direction > 0 && vel.x < x )
					vel.x += game.frameDeltaTime * 50;
				else if( direction < 0 && vel.x > x )
					vel.x -= game.frameDeltaTime * 50;
			}
			else
				vel.x = x + groundVel.x;
		}
		else
			vel.x = groundVel.x;

		if( dy != 0 )
			vel.y = dy + groundVel.y;
		entity.velocity = vel;

		cam.Update(entity);
	}

	void onCollide(const CEntity @object, const vector2 &in position, const vector2 &in normal, float approachSpeed)
	{
		if( object !is null )
		{
			if( object.type == "zombie" )
			{
				if( normal.y > 0.5 )
				{
					// Make the player jump/bounce off the zombie's head
					entity.velocity = vector2(entity.velocity.x, jumpSpeed);
				}
				else
				{
					if( !isInvincible )
						TakeLife();
				}
			}
		}
	}

	void TakeLife()
	{
		// Take a life
		uint lives;
		game.GetProperty('lives', lives);
		game.SetProperty('lives', --lives);
		if( lives == 0 )
			game.EndLevel(endLevelFail);

		// Say something...
		entity.PlaySound('player_ouch',0.5);

		// For a while the player should not take more damage
		isInvincible = true;
		entity.SetTransparency(0.5);
		entity.SetTimer(invincibleTime, makeVulnerable);
	}

	void onTimer(int id)
	{
		switch( id )
		{
		case makeVulnerable:
			isInvincible = false;
			entity.SetTransparency(0);
			break;

		case canAttackAgain:
			isAttacking = false;
			break;
		}
	}

	CEntity @entity;
	bool isAttacking;  // is the player currently attacking?
	bool isInvincible; // is the player currently invincible?
	int direction;     // in which direction is the player looking?
}