const float walkSpeed = 5;
const float jumpSpeed = 10;

enum TimerId
{
  killTimer,
  checkForPlayer
}

class CZombie : IEntityController
{
	CZombie(CEntity @ent)
	{
		@entity = ent;
		isDying = false;
		seePlayer = false;
	}

	void onThink()
	{
		// Acquire the player object
		if( target is null )
			@target = game.FindEntityByName('player');

		// Check the direction in which the zombie should walk
		if( target !is null )
		{
			// Is the player close enough to care? and at eye level?
			if( (target.position - entity.position).length() < 20 &&
			    abs(target.position.y - entity.position.y) < 2 )
			{
				// Say something if we see the player now
				if( !seePlayer )
				{
					entity.PlaySound('zombie_groan');
					seePlayer = true;
				}
			
				// Can only walk if on the ground
				vector2 tmp;
				float f = game.RayCast(entity, 1, entity.position, entity.position - vector2(0, 1.01), null, tmp);
				if( f < 1 )
				{
					if( target.position.x < entity.position.x - 0.5f )
					{
						entity.velocity = vector2(-walkSpeed, entity.velocity.y);
						entity.SetAnimation("walk_l");
					}
					else if( target.position.x > entity.position.x + 0.5f )
					{
						entity.velocity = vector2(walkSpeed, entity.velocity.y);
						entity.SetAnimation("walk_r");
					}
				}
			}
			else
			{
				// Stop walking if on the ground
				vector2 tmp;
				float f = game.RayCast(entity, 1, entity.position, entity.position - vector2(0, 1.01), null, tmp);
				if( f < 1 )
					entity.velocity = vector2(0, entity.velocity.y);
				seePlayer = false;
			}

			if( abs(entity.velocity.x) < 0.01 )
				entity.SetAnimation("idle");

			// Sleep a while
			entity.SetTimer(0.25, checkForPlayer);
		}
	}

	void onCollide(const CEntity @object, const vector2 &in position, const vector2 &in normal, float approachSpeed)
	{
		// If the player is jumping on me, then I die
		if( object !is null && object.type == 'player' && normal.y < -0.5f )
		{
			// Kill myself
			Die();
		}
		else if( object !is null )
		{
			// Stop moving
			entity.velocity = vector2(0,0);
			entity.SetAnimation("idle");
		}
	}

	void onTouch(const CEntity @origin)
	{
		// Who's touching me?
		if( origin !is null and origin.type == 'player' )
		{
			// Ouch, the player hit me...
			// Jumping back to avoid more hits
			if( origin.position.x < entity.position.x )
				entity.velocity = vector2(jumpSpeed, jumpSpeed);
			else
				entity.velocity = vector2(-jumpSpeed, jumpSpeed);
		}
	}

	void onTimer(int id)
	{
		if( id == killTimer )
			entity.Kill();
		else if( id == checkForPlayer && !isDying )
			entity.awake = true;
	}

	void Die()
	{
		entity.velocity = vector2(0,0);  // No movement
		entity.SetAnimation("death");    // Death animation
		// TODO: Instead of removing collisions and gravity, the 
		//       collision shape should be flatten to the ground. 
		//       This will allow the body to accompany moving 
		//       platforms.
		entity.SetCollision(false);      // No more collisions
		entity.SetGravity(vector2(0,0)); // No gravity
		entity.awake = false;            // Stop thinking

		isDying = true;	

		// Set a timer after which we will actually kill the object
		entity.SetTimer(0.4f, killTimer);
	}

	const CEntity @target;
	CEntity       @entity;
	bool           isDying;
	bool           seePlayer;
}