#include "common.as"

CGuiElement @mouse;

// This handler is called whenever the menu should be shown
// id = 0 when called to show the main menu, or when the game is paused
// Other id's are called from the scripts to allow custom menu options
void onShowMenu(uint id)
{
	// Set up the gui elements for the start menu
	CreateMouse();

	CGuiElement @bkgd = gui.CreateElement(vector2(0,0), vector2(1024,768), null);
	bkgd.srcImageIndex = gui.LoadImage("menu_gradient.png");
	
	// Create the menu options
	if( id == 0 )
	{
		// Main menu
		
		// Create background image
		CGuiElement @knight = gui.CreateElement(vector2(50,30), vector2(428,702), null);
		knight.srcImageIndex = gui.LoadImage("menu_knight.png");

		if( game.isGameOn )
		{
			CreateButton(712,500, 'Resume game', onResume);
			CreateButton(712,430, 'Restart level', onRestartLevel);
		}
		CreateButton(712,360, 'Start new game', onStart);
		CreateButton(712,290, 'Quit game', onQuit);
	}
	else if( id == 1 )
	{
		// The level failed

		CreateLabel(512,480, 'You failed!');
		CreateButton(512,380, 'Retry level', onRestartLevel);
		CreateButton(512,310, 'Back to menu', onBackToMenu);
	}
	else if( id == 2 )
	{
		// The level succeeded
		
		CreateLabel(512,580, 'You completed the level!');
		CreateButton(512,480, 'Continue', onRestartLevel);
		CreateButton(512,410, 'Back to menu', onBackToMenu);
	}
	else if( id == 3 )
	{
		// The game has been completed
		
		CreateLabel(512,580, 'The End');
		CreateButton(512,410, 'Back to menu', onBackToMenu);
	}
}

void onMouseOver(bool isMouseOver, CGuiElement @bt)
{
	if( isMouseOver )
	{
		bt.KillTimer(0);
		bt.SetTimer(0, 1);
	}
	else
	{
		bt.KillTimer(1);
		bt.SetTimer(0, 0);
	}
}

void onStart(CGuiElement @)
{
	game.StartNewGame();
}

void onQuit(CGuiElement @)
{
	game.QuitGame();
}

void onResume(CGuiElement @)
{
	game.ResumeGame();
}

void onRestartLevel(CGuiElement @)
{
	game.RestartLevel();
}

void onBackToMenu(CGuiElement @)
{
	// EndGame() finishes the game and shows the main menu
	game.EndGame();
}

void onTimer(int id, CGuiElement @el)
{
	if( id == 1 && el.scale.x < 1.1 )
	{
		el.scale += vector2(0.05, 0.05);
		el.SetTimer(0.01, 1);
		if( el.scale.x > 1.1 )
			el.scale = vector2(1.1, 1.1);
	}
	else if( id == 0 && el.scale.x > 1.0 )
	{
		el.scale -= vector2(0.025, 0.025);
		el.SetTimer(0.01, 0);
		if( el.scale.x < 1.0 )
			el.scale = vector2(1.0, 1.0);
	}
}

void CreateMouse()
{
	@mouse = @gui.CreateElement(vector2(32,32), vector2(32,32));
	mouse.center = vector2(16,24);
	gui.SetMouseElement(mouse);
	mouse.srcImageIndex = gui.LoadImage("arrow.png");
	mouse.zPosition = -1000; // Make sure the cursor is in front of all other elements
}

CGuiElement @CreateButton(float x, float y, const string &in text, OnClickHandler @handler)
{
	CGuiElement @bt = gui.CreateElement(vector2(x,y), vector2(50,50), null);
	bt.fontIndex    = gui.LoadFont("menu.fnt");
	bt.text         = text;
	@bt.onClick     = handler;
	@bt.onMouseOver = onMouseOver;
	@bt.onTimer     = onTimer;
	return @bt;
}

CGuiElement @CreateLabel(float x, float y, const string &in text)
{
	// TODO: Labels should have different colors than the buttons
	CGuiElement @el = gui.CreateElement(vector2(x,y), vector2(50,50), null);
	el.fontIndex = gui.LoadFont("menu.fnt");
	el.text      = text;
	el.color     = 0xFFFFFF00;
	return @el;
}


